/*
 *  LibRoadnav
 *  MapAppearanceSettings.h
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Class used to manage detail settings
///
//////////////////////////////////////////////////////////////////////////////

#ifndef MAPDETAILSETTINGS_H
#define MAPDETAILSETTINGS_H

#include <wx/config.h>

#include "MapControlDataEntry.h"
#include "DetailSettings.h"

#include <vector>
#include <map>

enum EMiscColor
{
	MiscColorBackground = 0,
	MiscColorForeground,
	MiscColorMarkerBackground,
	MiscColorMarkerForeground
};

#define LAST_MISCCOLOR		MiscColorMarkerForeground

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Class used to manage detail settings
///
//////////////////////////////////////////////////////////////////////////////
class MapAppearanceSettings
{
	public: // public functions
		MapAppearanceSettings();

		void ClearDetailLevels();
		void AddDetailLevel(double scale);

		wxInt16 GetDetailLevelCount() const;
		
		const DetailSettings& GetDetailSettings(wxInt16 iDetailLevel, ERecordType eRT) const;
		DetailSettings& GetDetailSettings(wxInt16 iDetailLevel, ERecordType eRT);
		
		double GetScale(wxInt16 iDetailLevel) const;
		wxInt16 GetBestDetailLevel(double scale) const;

		void SetMiscColor(EMiscColor eIndex, const wxColour & clrColor);
		const wxColour & GetMiscColor(EMiscColor eIndex) const;

		void SetLineColor(ERecordType eRecordType, const wxColour & clrColor, int iDetailLevel = -1);
		void SetTextColor(ERecordType eRecordType, const wxColour & clrColor, int iDetailLevel = -1);
		void SetColor(ERecordType eRecordType, const wxColour & clrColor, int iDetailLevel = -1);
		
		void FixTextColorInsideLines();

		void LoadDefaults();

		bool Load(wxConfigBase* pConfig);
		void Save(wxConfigBase* pConfig) const;
		
	private: // private data
		/// STL map containing information on the various detail levels
		std::map<wxInt16,DetailSettings> m_mapDetailSettings;

		/// Background color of control (land)
		std::map<EMiscColor, wxColour> m_mapMiscColors;

		/// STL map from detail level to scale level. This scaling level
		/// actually zooms the map in/out. It does not affect the size
		/// of everything else.
		std::vector<double> m_vDetailLevelToScale;
};

#endif
