/*
 *  LibRoadnav
 *  MapControlData.h
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Interface used by MapControl to access the record set
///
//////////////////////////////////////////////////////////////////////////////

#ifndef MAPCONTROLDATA_H
#define MAPCONTROLDATA_H

#include <wx/datetime.h>

#include <map>
#include <vector>

#include "Point.h"
#include "MapControlDataEntry_Std.h"
#include "MapControlDataImporter.h"
#include "ProgressDialog.h"

typedef std::vector<wxUint32> vIntArray;
typedef std::pair<int,int> tEdge;
typedef std::vector<int> tVectorOfInts;

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Interface used by MapControl to access mapping data
///
//////////////////////////////////////////////////////////////////////////////
class IMapControlData
{
	public: // public functions

		virtual ~IMapControlData() = 0;
	
		virtual void Clear() = 0;

		virtual int Count() = 0;
		virtual IMapControlDataEntry * GetRecord(wxUint32 iIndex) = 0;
		virtual IMapControlDataEntry * GetRecordByID(const tRecordID & idRec) = 0;
	
		virtual std::vector<tRecordID> ShortestPath(const Point & ptStart, const Point & ptEnd, const tRecordID & idStart, const tRecordID & idEnd, bool bProgressDialog = true) = 0;

		virtual std::vector<Point> TranslateRecordsToCoordinates(const std::vector<tRecordID> & vRecords) = 0;
		virtual wxString TranslateRecordsToDirections(const std::vector<tRecordID> & vRecords) = 0;
		
		virtual std::vector<IMapControlDataEntry *> IntersectionInformation(const tRecordID & iPreviousRec, const tRecordID & iRec, double * pfDeltaAngle, wxString *  pstrDirectionChange) = 0;

		virtual void LoadRegion(const Point & ptTopLeft, const Point & ptBottomRight, const bool * pbVisibility, bool bClearExisting = true) = 0;

		virtual bool Save(MapControlDataImporter * pImporter, ProgressDialog * pdlgProgress = NULL, int iProgressStart = 0, int iProgressTicks = 1) = 0;
	
		virtual wxInt16 GetNumRecordTypes() = 0;
		virtual wxString GetRecordTypeDisplayName(wxInt16 iRecordType) = 0;

		virtual wxDateTime GetLastUpdated() = 0;

	public: // public member variables
	
};

#endif
