/*
 *  LibRoadnav
 *  MapControlDataEntry_Std.h
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Contains MapControlDataEntry_Std - a class for a single type 1 feature or equivalent
/// in the database.
///
//////////////////////////////////////////////////////////////////////////////

#ifndef _MAPRECORDENTRY_H
#define _MAPRECORDENTRY_H

#include <wx/wx.h>

#include "MapControlDataEntry.h"

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Implements IMapControlDataEntry
///
//////////////////////////////////////////////////////////////////////////////
class MapControlDataEntry_Std : public IMapControlDataEntry
{
	public:
		// giving this class a constructor results in a horrible performance
		// hit in new() when no optimizations are used
		virtual bool IsRoad() const;
		virtual double CostFactor() const;
		virtual wxString SmartName() const;
		virtual bool IsOneWay() const;
		virtual double TimeFactor() const;
		virtual double RecordDistance() const;
		virtual bool IsSameRoad(IMapControlDataEntry * psRec) const;
		virtual wxString NameAndType(int iEntry = 0, bool bLongType = false) const;
		virtual double Distance(const Point & ptSearch, int * piBestSegment = NULL) const;
		virtual bool ContainsPoint(const Point & ptCheck) const;
		virtual bool IntersectsRect(const Point & ptTopLeft, const Point & ptBottomRight) const;
		virtual bool IsRegion() const;
		virtual wxInt32 CountyCode() const;
		virtual wxString Dump() const;
		
		virtual void Clear();

		virtual void Serialize(FILE * hFile, bool bSave, void * pvArg);

		bool operator <(const MapControlDataEntry_Std & x) const;
		bool operator >(const MapControlDataEntry_Std & x) const;
		bool operator <=(const MapControlDataEntry_Std & x) const;
		bool operator >=(const MapControlDataEntry_Std & x) const;
		bool operator ==(const MapControlDataEntry_Std & x) const;
		
		static void SetRoutingPreferences(double fPreferSmallRoads, double fPreferLargeRoads, double fPreferHighways);
		static void GetRoutingPreferences(double * pfPreferSmallRoads, double * pfPreferLargeRoads, double * pfPreferHighways);
		
	private:
		static double m_fPreferSmallRoads;
		static double m_fPreferLargeRoads;
		static double m_fPreferHighways;
};

#endif /* _MAPRECORDENTRY_H */
