/*
 *  LibRoadnav
 *  MapControlDataImporter_OSM.h
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Imports OpenStreetMap data into the Roadnav tile-layers
///
//////////////////////////////////////////////////////////////////////////////

#ifndef MAPCONTROLDATAIMPORTER_OSM_H
#define MAPCONTROLDATAIMPORTER_OSM_H

#include <wx/datetime.h>

#include "Constants.h"
#include "Point.h"
#include "MapControlDataEntry_Std.h"
#include "MapControlData.h"
#include "MapControlDataImporter.h"

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Imports TIGER/Line data from the USCB
///
//////////////////////////////////////////////////////////////////////////////
class MapControlDataImporter_OSM : public MapControlDataImporter
{
	public: // public functions
		MapControlDataImporter_OSM();
		virtual ~MapControlDataImporter_OSM();
	
		bool DownloadRegion(IMapControlData * pData, Point ptTL, Point ptBR, wxString strUsername, wxString strPassword, ProgressDialog * pdlgProgress, int iProgressStart, int iProgressTicks);
		
		virtual IMapControlDataEntry * GetRecord(wxUint32 iIndex);
		virtual int Count();

		bool IsLoaded(Point pt);

		bool Load(wxString strFilename);
		bool LoadDirect(wxString strFilename, IMapControlData * pMapControlData, ProgressDialog * pdlgProgress, int iProgressStart, int iProgressTicks, Point * ptCenter);

	private: // private functions
		bool DownloadSlice(IMapControlData * pData, Point ptTL, Point ptBR, wxString strUsername, wxString strPassword, ProgressDialog * pdlgProgress, int iProgressStart, int iProgressTicks);
		bool IsLoaded(Point ptTL, Point ptBR);
		void ClassifyType(const char * pszK, const char * pszV, ERecordType * peType);

		void Clear();

	private: // private member variables

		/// the records themselves
		MapControlDataEntry_Std * m_psRecords;
		
		/// the number of records
		int m_nRecords;
		
		/// How many degrees worth of data should be downloaded at once. Big number = long wait for each file, small number = too many files.
		double m_fDownloadGranularity;
};

#endif
