/*
 *  LibRoadnav
 *  MapControlDataImporter_TigerLine.h
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Imports Tiger/Line data into the Roadnav tile-layers
///
//////////////////////////////////////////////////////////////////////////////

#ifndef MAPCONTROLDATAIMPORTER_TIGERLINE_H
#define MAPCONTROLDATAIMPORTER_TIGERLINE_H

#include <wx/datetime.h>

#include <map>
#include <vector>

#include "Constants.h"
#include "Point.h"
#include "MapControlDataEntry_Std.h"
#include "MapControlData.h"
#include "MapControlDataImporter.h"

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Imports TIGER/Line data from the USCB
///
//////////////////////////////////////////////////////////////////////////////
class MapControlDataImporter_TigerLine : public MapControlDataImporter
{
	public: // public functions
		MapControlDataImporter_TigerLine(char * pszSource = NULL);
		virtual ~MapControlDataImporter_TigerLine();
	
		static bool IsCountyLoaded(wxInt32 iStateCountyCode);

		virtual bool DownloadCounties(IMapControlData * pData, std::vector<wxInt32> viStateCountyCodes, ProgressDialog * pdlgProgress = NULL, int iProgressStart = 0, int iProgressTicks = 1);
		virtual bool DownloadCounty(IMapControlData * pData, wxInt32 iStateCountyCode, ProgressDialog * pdlgProgress = NULL, int iProgressStart = 0, int iProgressTicks = 1);
		
		virtual IMapControlDataEntry * GetRecord(wxUint32 iIndex);
		virtual int Count();

		void SetCompactMapsMode(bool bCompactMaps = true);

	private: // private functions
		bool Load(wxString strBaseName);

		bool LoadType1(wxString strFilename);
		bool LoadType2(wxString strFilename);
		bool LoadType4(wxString strFilename);
		bool LoadType5(wxString strFilename);
		bool LoadType6(wxString strFilename);
		bool LoadTypeI(wxString strFilename);
		bool LoadTypeP(wxString strFilename);

		void CalculateNameTypeInstances();
		void CalculateSmartNames();
		wxString CalculateOneSmartName(int iRecord);

		void Clear();

		void SortRecords(int left, int right);
	
		void FixZipCodes();
	

	private: // private member variables

		/// the records themselves
		MapControlDataEntry_Std * m_psRecords;
		
		/// the number of records
		int m_nRecords;

		/// true = save disk space and memory by not loading type 2 records (reduces quality)
		/// false = high quality, high memory and disk space
		bool m_bCompactMaps;

		// some temporary variables used during the loading process
		std::map<wxUint32,wxUint32> m_mapTLIDtoRecord;
		std::map<wxUint32,wxUint32> m_mapPolyIDtoRecord;
		std::map<wxUint32,wxUint32> m_mapPolyIDtoSide;
		std::map<wxUint32,vIntArray> m_mapAdditionalNameIDtoRecord;

		/// the number of instances each feature name + type has been seen
		/// used to figure out the most common name for roads with multiple
		/// names (e.g. Durham Road + Main St. + Route 108, etc.)
		std::map<wxString,int> m_mapFeatureNameTypeInstances;
};

#endif
