/*
 *  LibRoadnav
 *  MapControlData_Foundation.h
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Manages the Tiger/Line data.
///
//////////////////////////////////////////////////////////////////////////////

#ifndef MAPCONTROLDATA_FOUNDATION_H
#define MAPCONTROLDATA_FOUNDATION_H

#include <wx/datetime.h>

#include <map>
#include <vector>

#include "Point.h"
#include "MapControlDataEntry.h"
#include "MapControlData.h"

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Provides some generic MapControlData functionality
///
//////////////////////////////////////////////////////////////////////////////
class MapControlData_Foundation : public IMapControlData
{
	public: // public functions
		MapControlData_Foundation();
		virtual ~MapControlData_Foundation();
	
		virtual void Clear();

		virtual IMapControlDataEntry * GetRecord(wxUint32 iIndex);
		virtual IMapControlDataEntry * GetRecordByID(const tRecordID & idRec);
		virtual int Count();
	
		virtual std::vector<tRecordID> ShortestPath(const Point & ptStart, const Point & ptEnd, const tRecordID & idStart, const tRecordID & idEnd, bool bProgressDialog = true);

		virtual std::vector<Point> TranslateRecordsToCoordinates(const std::vector<tRecordID> & vRecords);
		virtual wxString TranslateRecordsToDirections(const std::vector<tRecordID> & vRecords);
		
		virtual std::vector<IMapControlDataEntry *> IntersectionInformation(const tRecordID & iPreviousRec, const tRecordID & iRec, double * pfDeltaAngle, wxString * pstrDirectionChange);

		virtual wxInt16 GetNumRecordTypes();
		virtual wxString GetRecordTypeDisplayName(wxInt16 iRecordType);

	protected:

	private: // private functions
		void EnumerateVertices();
		void ClearVertices();

	public: // public member variables
		

	protected:
		/// the records themselves
		MapControlDataEntry_Std * m_psRecords;
		
		/// the number of records
		int m_nRecords;

	private: // private member variables
		std::map<Point,tVectorOfInts> m_mapCoordinateToRecords;
};

#endif
