/*
 *  LibRoadnav
 *  MapMarker.h
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Encompasses the MapMarker class - describes a marker on the map
///
//////////////////////////////////////////////////////////////////////////////

#ifndef _MAPMARKER_H
#define _MAPMARKER_H

#include <wx/wx.h>

#include <vector>

#include "Point.h"
#include "Address.h"

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Describes a marker on the map
///
//////////////////////////////////////////////////////////////////////////////
class MapMarker
{
	public:
		MapMarker();

		MapMarker(	const wxString & strName, 
					Address * psAddr, 
					double fHeading = -1, 
					bool bCalloutBox = false, 
					bool bSmallLabel = true, 
					bool bArrow = true, 
					const wxString & strIcon = wxT(""),
					bool bScaleIcon = true,
					bool bVisible = true);

		void Init(	const wxString & strName, 
					Address * psAddr, 
					double fHeading = -1, 
					bool bCalloutBox = false, 
					bool bSmallLabel = true, 
					bool bArrow = true, 
					const wxString & strIcon = wxT(""),
					bool bScaleIcon = true,
					bool bVisible = true);
			
		wxString ToString();
		int FromString(wxString strDesc);

		int Draw(class MapControl * pCtl, wxDC * dc, double fDrawingScaleFactor, bool bScreen);
		int DrawIcon(class MapControl * pCtl, wxDC * dc, double fDrawingScaleFactor, bool bScreen);
		int DrawCalloutBox(class MapControl * pCtl, wxDC * dc, double fDrawingScaleFactor, bool bScreen);
		int DrawSmallLabel(class MapControl * pCtl, wxDC * dc, double fDrawingScaleFactor, bool bScreen);
		int DrawArrow(class MapControl * pCtl, wxDC * dc, double fDrawingScaleFactor, bool bScreen);
	
		/// Name of this marker
		wxString m_strName;
		
		/// Text to be shown in the marker
		wxString m_strText;
		
		/// Marker heading (for GPS marker)
		double m_fHeading;
		
		/// Marker long/lat
		Address m_cAddress;

		/// true = callout box visible, false = no callout box
		bool m_bCalloutBox;

		/// true = small semitransparent label visible, false = no small label
		bool m_bSmallLabel;

		/// true = draw arrow if m_fHeading >= 0, false = no arrow
		bool m_bArrow;
		
		/// true = scale icon down as zoom out, false = no scaling
		bool m_bScaleIcon;

		/// Filename to the icon for this marker, or an empty string for no icon
		wxString m_strIcon;
		
		/// Visibility
		bool m_bVisible;
};


#endif
