/*
 *  LibRoadnav
 *  MapPolygon.h
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Contains the MapPolygon class - a class that contains an open or closed
/// polygon.
///
//////////////////////////////////////////////////////////////////////////////

#ifndef MAPPOLYGON_H
#define MAPPOLYGON_H

#include <vector>

#include "Point.h"

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Represents an open or closed polygon.
///
//////////////////////////////////////////////////////////////////////////////
class MapPolygon
{
	public:
		MapPolygon();
		MapPolygon(Point * pptPoints, int nPoints);
		MapPolygon(const MapPolygon & cCopy);

		const MapPolygon & operator =(const MapPolygon & cCopy);

		~MapPolygon();
		void Clear();
		
		Point GetPoint(int n);
		int GetSize();
		Point GetBeginning();
		Point GetEnd();
		
		bool IsAdjacentTo(MapPolygon cPolygon);
		bool IsEmpty();
		bool IsClosed();
		
		bool HasVertex(Point pt);
		bool HasPolygon(MapPolygon cPolygon);

		bool Attach(MapPolygon cPolygon);

		double GetArea();
		
		bool ContainsPoint(Point ptCheck);
		
	private:
		void AllocatePoints(int nPointsNew);

		/// Array of vertices
		Point * m_pPoints;

		/// Number of vertices
		int m_nPoints;
};

MapPolygon CombinePolygons(std::vector<MapPolygon> vPolygons, Point ptInside, MapPolygon cChain);

#endif
