/*
 *  LibRoadnav
 *  MapRepresentations.h
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Functions related to representing colors, positions, distances, times,
/// addresses, etc. in a string. It also contains functions to parse such
/// strings back into their original form.
///
//////////////////////////////////////////////////////////////////////////////

#ifndef MAPREPRESENTATIONS_H
#define MAPREPRESENTATIONS_H

#include <wx/wx.h>

#include "Point.h"
#include "Address.h"

//////////////////////////////////////////////////////////////////////////////
/// Color representation
//////////////////////////////////////////////////////////////////////////////
wxString FormatColor(const wxColour & c);
wxColour ParseColor(const wxString & strColor);

//////////////////////////////////////////////////////////////////////////////
/// File size representation
//////////////////////////////////////////////////////////////////////////////
wxString FormatFileSize(wxUint32 uiSize);

//////////////////////////////////////////////////////////////////////////////
/// Distance representation
//////////////////////////////////////////////////////////////////////////////

wxString FormatDistance(double fDistance);

//////////////////////////////////////////////////////////////////////////////
/// Time representation
//////////////////////////////////////////////////////////////////////////////

wxString FormatTime(double fTime);

//////////////////////////////////////////////////////////////////////////////
/// Name/abbreviation representation
//////////////////////////////////////////////////////////////////////////////

wxString FormatName(const wxString & strIn);
wxString FormatAbbreviation(const wxString & strIn);

//////////////////////////////////////////////////////////////////////////////
/// Direction representation
//////////////////////////////////////////////////////////////////////////////

wxString FormatHeading(double fHeading);

//////////////////////////////////////////////////////////////////////////////
/// Speed representation
//////////////////////////////////////////////////////////////////////////////

wxString FormatSpeed(double fSpeed);

//////////////////////////////////////////////////////////////////////////////
/// SI vs. US unit representation
//////////////////////////////////////////////////////////////////////////////

enum EUnits
{
	USUnits, 
	SIUnits
};

void SetDisplayUnits(EUnits eUnits);
EUnits GetDisplayUnits();
 
#endif
