/*
 *  LibRoadnav
 *  MapState.h
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Maps states to/from the FIPS code and abbreviation.
///
//////////////////////////////////////////////////////////////////////////////

#ifndef _MAPSTATE_H
#define _MAPSTATE_H

#include <vector>

#include "AutoComplete.h"

bool LoadStateInformation(const wxString & strFilename);

const wxString & StateAbbreviationByCode(int iCode);
wxString StateAbbreviationByName(const wxString & strName);
const wxString & StateNameByCode(int iStateCode);
const wxString & StateNameByAbbreviation(const wxString & strName);
int StateCodeByName(const wxString & strStateName);
wxString ClosestStateName(const wxString & strState);

std::vector<wxString> GetStateAbbreviations();
std::vector<wxString> GetStateNames();

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Autocompletes a state name
///
//////////////////////////////////////////////////////////////////////////////
class AutoComplete_State : public IAutoComplete
{
	public:
		AutoComplete_State();
		virtual wxString ClosestMatch(const wxString & strInput);
		virtual int GetUpdateDelay();
};

#endif /* _MAPSTATE_H */
