/*
 *  LibRoadnav
 *  MapTrack.h
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// This file contains the MapTrack class which represents one track or path
/// on the MapControl
///
//////////////////////////////////////////////////////////////////////////////

#ifndef _MAPTRACK_H
#define _MAPTRACK_H

#include <wx/wx.h>

#include <vector>

#include "Point.h"

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Represents one track/path on the MapControl, along with related
/// attributes
///
//////////////////////////////////////////////////////////////////////////////
class MapTrack
{
	public:
		MapTrack();
		MapTrack(const wxString & strName, const std::vector<Point> & vptCoordinates, const wxColour & clrTrack, bool bAffectsAutoscale);

		void Init(const wxString & strName, const std::vector<Point> & vptCoordinates, const wxColour & clrTrack, bool bAffectsAutoscale);
			
		int Draw(class MapControl * pCtl, wxDC * dc, double fDrawingScaleFactor);

		/// Name of this track
		wxString m_strName;
		
		/// Track coordinates
		std::vector<Point> m_vptCoordinates;

		/// Track color
		wxColour m_clrTrack;

		/// Should this track affect autoscaling?
		bool m_bAffectsAutoscale;
};


#endif
