/*
 *  LibRoadnav
 *  StringTable.h
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Contains StringTable class
///
//////////////////////////////////////////////////////////////////////////////

#ifndef STRINGTABLE_H
#define STRINGTABLE_H

#include <wx/wx.h>

#define INVALID_STRING_ENTRY	0xffffffff

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Implements a table of commonly used strings. IMapControlDataEntry
/// objects can point to this table rather than having their own private
/// copy of each string name.
///
//////////////////////////////////////////////////////////////////////////////
class StringTable
{
	public:
		StringTable(wxString strFilename);
		~StringTable();

		void Init();
		void Clear();
		void Load();		
		void Save();
		
		wxUint32 GetEntryIndex(const char * pszName, bool bCreate);
		char * GetEntryPointer(wxUint32 uiEntry);
		char * GetEntryPointer(const char * pszName, bool bCreate);
		
	private:
		/// Table data itself
		char ** m_ppszTable;
		
		/// Array indicating how to put the table in alphabetical order
		wxUint32 * m_puiSortKey;
		
		/// Actual memory allocated for m_ppszTable
		wxUint32 m_nTableMemorySize;
		
		/// Entries in m_ppszTable in use
		wxUint32 m_nTableEntriesInUse;
		
		/// Number of entries m_ppszTable should grow by when array has to be expanded
		wxUint32 m_nGrowBy;
		
		/// Indicates if the table has been modified since last Load/Save.
		bool m_bModified;
		
		/// Filename of the table.
		wxString m_strFilename;
};

#endif
