/*
 *  LibRoadnav
 *  DetailSettings.cpp
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Contains the DetailSettings class, a class representing the various
/// settings for a particular detail level and feature type (e.g. line width,
/// text color, etc.)
///
//////////////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef _MSC_VER
#pragma warning(disable: 4786)
#endif

#include <wx/wx.h>

#include "DetailSettings.h"
#include "MapRepresentations.h"

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Constructor - initialize everything
///
//////////////////////////////////////////////////////////////////////////////
DetailSettings::DetailSettings()
{
	m_clrLine = *wxBLACK;
	m_iWidth = 1;
	m_iStyle = wxSOLID;
	m_bFastLine = false;
	m_clrText = *wxBLACK;
	m_iFontSize = 8;
	m_bTextVisible = true;
	m_bLineVisible = true;
	m_iArrowSpacing = 0;
	m_bRailroadStyle = false;
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Save this object to the supplied configuration object
///
//////////////////////////////////////////////////////////////////////////////
void DetailSettings::Save(wxConfigBase * pConfig) const
{
	pConfig->Write(wxT("Version"), 1);
	pConfig->Write(wxT("LineColor"), FormatColor(m_clrLine));
	pConfig->Write(wxT("Width"), m_iWidth);
	pConfig->Write(wxT("Style"), m_iStyle);
	pConfig->Write(wxT("FastLine"), m_bFastLine);
	pConfig->Write(wxT("TextColor"), FormatColor(m_clrText));
	pConfig->Write(wxT("FontSize"), m_iFontSize);
	pConfig->Write(wxT("TextVisible"), m_bTextVisible);
	pConfig->Write(wxT("LineVisible"), m_bLineVisible);
	pConfig->Write(wxT("RailroadStyle"), m_bRailroadStyle);
	pConfig->Write(wxT("ArrowSpacing"), m_iArrowSpacing);
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Load this object from the supplied configuration object
///
//////////////////////////////////////////////////////////////////////////////
bool DetailSettings::Load(wxConfigBase * pConfig)
{
	wxString strTmp;
	int iRegVersion = 0;

	pConfig->Read(wxT("Version"), &iRegVersion);
	
	strTmp = pConfig->Read(wxT("LineColor"), wxT(""));
	if (strTmp != wxT(""))
		m_clrLine = ParseColor(strTmp);
	else
		return true;
		
	pConfig->Read(wxT("Width"), &m_iWidth);
	
	// ignore style for version 0 reg keys
	if (iRegVersion >= 1)
		pConfig->Read(wxT("Style"), &m_iStyle);
		
	pConfig->Read(wxT("FastLine"), &m_bFastLine);
	
	strTmp = pConfig->Read(wxT("TextColor"), wxT(""));
	if (strTmp != wxT(""))
		m_clrText = ParseColor(strTmp);
	else
		return true;
	
	pConfig->Read(wxT("FontSize"), &m_iFontSize);
	pConfig->Read(wxT("TextVisible"), &m_bTextVisible);
	pConfig->Read(wxT("RailroadStyle"), &m_bRailroadStyle);
	pConfig->Read(wxT("LineVisible"), &m_bLineVisible);
	pConfig->Read(wxT("ArrowSpacing"), &m_iArrowSpacing);
	
	return false;
}

