/*
 *  LibRoadnav
 *  MapTrack.cpp
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// This file contains the MapTrack class which represents one track or path
/// on the MapControl
///
//////////////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef _MSC_VER
#pragma warning(disable: 4786)
#endif

#include <wx/wx.h>

#include "MapTrack.h"
#include "MapControl.h"
#include "MapDrawing.h"
#include "MapSupport.h"

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Constructor - do nothing
///
//////////////////////////////////////////////////////////////////////////////
MapTrack::MapTrack()
{
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Constructor - initialize track object
///
//////////////////////////////////////////////////////////////////////////////
MapTrack::MapTrack(const wxString & strName, const std::vector<Point> & vptCoordinates, const wxColour & clrTrack, bool bAffectsAutoscale)
{
	Init(strName, vptCoordinates, clrTrack, bAffectsAutoscale);
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Initialize track object
///
//////////////////////////////////////////////////////////////////////////////
void MapTrack::Init(const wxString & strName, const std::vector<Point> & vptCoordinates, const wxColour & clrTrack, bool bAffectsAutoscale)
{
	m_strName = strName;
	m_vptCoordinates = vptCoordinates;
	m_clrTrack = clrTrack;
	m_bAffectsAutoscale = bAffectsAutoscale;
}
	
//////////////////////////////////////////////////////////////////////////////
///
/// \brief Draw this track onto MapControl pCtl, using context dc, and
/// scaling factor fDrawingScaleFactor
///
//////////////////////////////////////////////////////////////////////////////
int MapTrack::Draw(class MapControl * pCtl, wxDC * dc, double fDrawingScaleFactor)
{
	int iWidth;
	wxPoint ptTL(0, 0);
	wxPoint ptBR(pCtl->m_iControlWidth, pCtl->m_iControlHeight);
	
	iWidth = (int) (2 * fDrawingScaleFactor);

	unsigned int iSeg;
	for (iSeg = 1; iSeg < m_vptCoordinates.size(); iSeg++)
	{
		wxPoint pt1(pCtl->MapLongLatToScreen(m_vptCoordinates[iSeg - 1]));
		wxPoint pt2(pCtl->MapLongLatToScreen(m_vptCoordinates[iSeg]));

		if (LineIntersectsRect(ptTL, ptBR, pt1, pt2))
			DrawLine(dc, pt1.x, pt1.y, pt2.x, pt2.y, iWidth, m_clrTrack, wxSOLID);
	}
	return 0;
}
