/*
 *  LibRoadnav
 *  ProgressDialog.cpp
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// This file contains the ProgressDialog class, which
/// shows a dialog box containing a gauge and text control.
///
//////////////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <wx/wx.h>

#include "ProgressDialog.h"

BEGIN_EVENT_TABLE(ProgressDialog, wxDialog)
END_EVENT_TABLE()

//////////////////////////////////////////////////////////////////////////////
///
/// \brief ProgressDialog constructor
///
/// Creates the various controls in the dialog box
///
//////////////////////////////////////////////////////////////////////////////
ProgressDialog::ProgressDialog(wxWindow *parent, wxString strTitle, int iMax)
             : wxDialog(parent, -1, strTitle, wxDefaultPosition, wxDefaultSize, wxCAPTION)
{
    wxBoxSizer *sizerWnd = new wxBoxSizer(wxVERTICAL);

	//////////////////////////////////////////////////////////////////////////////
	// Status text
	//////////////////////////////////////////////////////////////////////////////

	m_ctlLabel = new wxStaticText(this, -1, wxT(""), wxDefaultPosition, wxSize(600, 20));
	
	sizerWnd->Add(
					m_ctlLabel,
					0, 
					wxGROW | wxALL,
					5);

	//////////////////////////////////////////////////////////////////////////////
	// Progres gauge
	//////////////////////////////////////////////////////////////////////////////

	m_ctlGauge = new wxGauge(this, -1, iMax);
	
	sizerWnd->Add(
					m_ctlGauge,
					0, 
					wxGROW | wxALL,
					5);

	//////////////////////////////////////////////////////////////////////////////
	// layout stuff
	//////////////////////////////////////////////////////////////////////////////
    SetSizer(sizerWnd);
	
    sizerWnd->Fit(this);

	Center();

	m_iPosition = 0;
	m_iRange = iMax;
	m_tLastUpdate = wxDateTime::Now();
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Move the progress bar
///
//////////////////////////////////////////////////////////////////////////////
void ProgressDialog::SetProgress(int iPosition)
{
	wxASSERT(iPosition >= 0);
	wxASSERT(iPosition <= m_iRange);
	wxASSERT(iPosition >= m_iPosition);

	// only update if the difference between the current position
	// and the new position is greater than 2% of the range of the gauge
	// or it has been 2 sec since the last update
	if (::abs(iPosition - m_iPosition) * 100.0 / m_iRange > 2 ||
		wxDateTime::Now() - m_tLastUpdate > wxTimeSpan(0, 0, 2, 0))
	{
		m_ctlGauge->SetValue(iPosition);
		m_iPosition = iPosition;
		
		m_tLastUpdate = wxDateTime::Now();

		wxSafeYield(this);
	}
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Change the status label
///
//////////////////////////////////////////////////////////////////////////////
void ProgressDialog::SetLabel(wxString strLabel)
{
	m_ctlLabel->SetLabel(strLabel);
	wxSafeYield(this);
}
