/*
 *  LibRoadnav
 *  Serialization.cpp
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */ 
 
//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Serialization base class. Derivative classes can be easily loaded
/// from/saved to a file.
///
//////////////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <wx/wx.h>

#include "Serialization.h"

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Destructor - do nothing
///
//////////////////////////////////////////////////////////////////////////////
Serialization::~Serialization()
{
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Serialize binary data to/from hFile.
///
//////////////////////////////////////////////////////////////////////////////
void Serialization::SerializeItem(FILE * hFile, bool bSave, char * i, int nLen)
{
	if (bSave)
		fwrite(i, sizeof(char), nLen, hFile);
	else
		fread(i, sizeof(char), nLen, hFile);
}


//////////////////////////////////////////////////////////////////////////////
///
/// \brief Serialize a boolean to/from hFile.
///
//////////////////////////////////////////////////////////////////////////////
void Serialization::SerializeItem(FILE * hFile, bool bSave, bool * i)
{
	if (bSave)
		fwrite(i, sizeof(bool), 1, hFile);
	else
		fread(i, sizeof(bool), 1, hFile);
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Serialize a double to/from hFile.
///
//////////////////////////////////////////////////////////////////////////////
void Serialization::SerializeItem(FILE * hFile, bool bSave, double * i)
{
	if (bSave)
		fwrite(i, sizeof(double), 1, hFile);
	else
		fread(i, sizeof(double), 1, hFile);
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Serialize a wxUint32 to/from hFile.
///
//////////////////////////////////////////////////////////////////////////////
void Serialization::SerializeItem(FILE * hFile, bool bSave, wxUint32 * i)
{
	if (bSave)
		fwrite(i, sizeof(wxUint32), 1, hFile);
	else
		fread(i, sizeof(wxUint32), 1, hFile);
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Serialize a wxUint16 to/from hFile.
///
//////////////////////////////////////////////////////////////////////////////
void Serialization::SerializeItem(FILE * hFile, bool bSave, wxUint16 * i)
{
	if (bSave)
		fwrite(i, sizeof(wxUint16), 1, hFile);
	else
		fread(i, sizeof(wxUint16), 1, hFile);
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Serialize a wxInt32 to/from hFile.
///
//////////////////////////////////////////////////////////////////////////////
void Serialization::SerializeItem(FILE * hFile, bool bSave, wxInt32 * i)
{
	if (bSave)
		fwrite(i, sizeof(wxInt32), 1, hFile);
	else
		fread(i, sizeof(wxInt32), 1, hFile);
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Serialize a wxInt16 to/from hFile.
///
//////////////////////////////////////////////////////////////////////////////
void Serialization::SerializeItem(FILE * hFile, bool bSave, wxInt16 * i)
{
	if (bSave)
		fwrite(i, sizeof(wxInt16), 1, hFile);
	else
		fread(i, sizeof(wxInt16), 1, hFile);
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Serialize a Serialization derived object to/from hFile.
///
//////////////////////////////////////////////////////////////////////////////
void Serialization::SerializeItem(FILE * hFile, bool bSave, Serialization * i, void * pvArg)
{
	i->Serialize(hFile, bSave, pvArg);
}
