/*
 *  LibRoadnav
 *  Terraserver.cpp
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Handles accessing the Terraserver map server
///
//////////////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef _MSC_VER
#pragma warning(disable: 4786)
#endif

#include <wx/wx.h>

#ifdef HAVE_MATH_H
#include <math.h>
#endif

#include "UTM.h"
#include "MapSupport.h"
#include "Point.h"
#include "URLs.h"

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Find the appropriate tile for coordinates fLong, fLat, at scaling level
/// iScale. Also retuns the coordinates of the edges of the tile.
///
//////////////////////////////////////////////////////////////////////////////
void GetTerraserverURL(int iScale, double fLong, double fLat, char * pszURL, double * pfLongLeft, double * pfLatTop, double * pfLongRight, double * pfLatBottom, int * piX, int * piY)
{
	double fNorthing;
	double fEasting;
	int iZone;
	int iType = 1; // 1 = aerial USGS, 2 = raster graphic, 4 = USGS natural color
	int iMultiplier;
	int iX;
	int iY;

	LongLatToUTM(fLong, fLat, &fEasting, &fNorthing, &iZone);

	iMultiplier = (int) (50 * pow((double) 2, iScale - 8));
	
	iX = (int) (fEasting / iMultiplier);
	iY = (int) (fNorthing / iMultiplier);

	UTMToLongLat((iX) * iMultiplier,
				 (iY + 1) * iMultiplier,
				 iZone,
				 pfLongLeft,
				 pfLatTop
				 );


	UTMToLongLat((iX + 1) * iMultiplier,
				 iY * iMultiplier,
				 iZone,
				 pfLongRight,
				 pfLatBottom
				 );


	sprintf(pszURL, "%s?T=%d&S=%d&X=%d&Y=%d&Z=%d",
																						static_cast<const char *>(GetURL(wxT("Terraserver")).mb_str(*wxConvCurrent)),
																						iType,
																						iScale,
																						iX,
																						iY,
																						iZone
																						);

	if (piX)
		*piX = iX;

	if (piY)
		*piY = iY;
};

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Get a terraserver tile for coordinates ptSearch, downloading if necessary
///
//////////////////////////////////////////////////////////////////////////////
bool FetchTerraserverTile(int iScale, Point ptSearch, char * pszFilename, Point * pptTL, Point * pptBR, bool bDownload)
{
	char szURL[256];
	char szFilename[64];
	int iX;
	int iY;
	double fTLLong;
	double fTLLat;
	double fBRLong;
	double fBRLat;

	GetTerraserverURL(iScale, ptSearch.GetLong(), ptSearch.GetLat(), szURL, &fTLLong, &fTLLat, &fBRLong, &fBRLat, &iX, &iY);
	pptTL->Set(fTLLong, fTLLat);
	pptBR->Set(fBRLong, fBRLat);

	sprintf(szFilename, "Terraserver%d.%d.%d.jpg", iX, iY, iScale);

	strcpy(pszFilename, GetDataPath(wxString(szFilename, *wxConvCurrent), bDownload ? wxString(szURL, *wxConvCurrent) : wxT(""), true).mb_str(*wxConvCurrent));

	return false;
}
