/*
 *  LibRoadnav
 *  URLs.cpp   
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// URL support functions
///
//////////////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef _MSC_VER
#pragma warning(disable: 4786)
#pragma warning(disable: 4800)
#endif

#include <wx/wx.h>

#include "URLs.h"

static wxConfigBase * g_pURLConfig = NULL;

static wxString g_astrURLs[] = 
{
	/// base URL for map data
	wxT("Tiger-Line Root"),
	wxT("http://www2.census.gov/geo/tiger/tiger2006se/,http://www2.census.gov/geo/tiger/tiger2007fe/,http://www2.census.gov/geo/tiger/tiger2007se/"),
	
	wxT("OpenStreetMaps"),
	wxT("http://api.openstreetmap.org/api/0.5/map"),

	/// base URL for landmark data in columnar format
/*	wxT("GNIS Root"),
	wxT("http://geonames.usgs.gov/geonames/stategaz/"),*/

	/// base URL for landmark data in delimited by pipe symbol format
	wxT("GNIS DECI Root"),
	wxT("http://geonames.usgs.gov/docs/stategaz/"),

	/// URL of state boundaries
	wxT("State Boundary Root"),
	wxT("http://www.census.gov/geo/cob/bdy/st/st00ascii/"),

	/// URL of county boundaries
	wxT("County Boundary Root"),
	wxT("http://www.census.gov/geo/cob/bdy/co/co00ascii/"),
	
	/// URL to pull down a list of counties
	wxT("Counties"),
	wxT("http://www.census.gov/geo/tigerline/app_a03.txt"),

	/// URL for the list of states, abbreviations, and FIPS codes
	wxT("States"),
	wxT("http://roadnav.sourceforge.net/states.tdf"),

	/// URL for the zip codes
	wxT("Zip Codes"),
	wxT("http://roadnav.sourceforge.net/zips.tdf"),

	/// URL for the Terraserver map server
	wxT("Terraserver"),
	wxT("http://terraserver-usa.com/tile.ashx"),

	/// URL for the abbreviations
	wxT("Abbreviations"),
	wxT("http://roadnav.sourceforge.net/abbreviations.txt"),

	// End marker
	wxT(""),
	wxT("")
};

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Set the configuration object that will store URLs from SetURL and
/// can be used to retrieve URLs for GetURL.
///
/// If NULL is specified, then SetURL will do nothing, and GetURL will
/// always return the default.
///
//////////////////////////////////////////////////////////////////////////////
void SetURLConfig(wxConfigBase * pURLConfig)
{
	g_pURLConfig = pURLConfig;
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Allows enumeration of the URLs. Returns the i'th URL name, or ""
/// if i >= (the number of URLs)
///
//////////////////////////////////////////////////////////////////////////////
wxString GetURLName(int i)
{
	int n;

	for (n = 0; g_astrURLs[n] != wxT(""); n += 2);

	if (i * 2 < n)
		return g_astrURLs[i * 2];

	return wxT("");
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Obtain the default URL for a URL name
///
//////////////////////////////////////////////////////////////////////////////
wxString GetDefaultURL(wxString strName)
{
	wxString strRtn;
	int i;

	strRtn = wxT("");

	for (i = 0; g_astrURLs[i] != wxT(""); i += 2)
	{
		if (strName == g_astrURLs[i])
			strRtn = g_astrURLs[i + 1];
	}

	return strRtn;
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Obtain the URL for a URL name. Custom URLs specified by SetURL
/// take precedence over the default.
///
//////////////////////////////////////////////////////////////////////////////
wxString GetURL(wxString strName)
{
	wxString strRtn;

	strRtn = wxT("");

	if (g_pURLConfig)
		g_pURLConfig->Read(wxString::Format(wxT("URL-%s"), strName.c_str()), &strRtn);

	if (strRtn == wxT(""))
		strRtn = GetDefaultURL(strName);

	return strRtn;
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Override the default URL for strName with strValue
///
//////////////////////////////////////////////////////////////////////////////
void SetURL(wxString strName, wxString strValue)
{
	if (!g_pURLConfig)
		return;

	if (GetDefaultURL(strName) == strValue)
		g_pURLConfig->Write(wxString::Format(wxT("URL-%s"), strName.c_str()), wxT(""));
	else
		g_pURLConfig->Write(wxString::Format(wxT("URL-%s"), strName.c_str()), strValue);
}
