/*
 *  LibRoadnav
 *  UniqueID.cpp
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Contains the UniqueID class - a class storing a 64 bit ID number
///
//////////////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <wx/wx.h>

#include "UniqueID.h"

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Destructor - does nothing
///
//////////////////////////////////////////////////////////////////////////////
UniqueID::~UniqueID()
{
}

//////////////////////////////////////////////////////////////////////////////
///
/// Less than comparison between two UniqueID objects
///
//////////////////////////////////////////////////////////////////////////////
bool UniqueID::operator < (const UniqueID & x) const
{
	if (m_iHigh < x.m_iHigh)
		return true;

	if (m_iHigh > x.m_iHigh)
		return false;

	return m_iLow < x.m_iLow;
}

//////////////////////////////////////////////////////////////////////////////
///
/// Greater than comparison between two UniqueID objects
///
//////////////////////////////////////////////////////////////////////////////
bool UniqueID::operator > (const UniqueID & x) const
{
	if (m_iHigh > x.m_iHigh)
		return true;

	if (m_iHigh < x.m_iHigh)
		return false;

	return m_iLow > x.m_iLow;
}

//////////////////////////////////////////////////////////////////////////////
///
/// Comparison operator between two UniqueID objects
///
//////////////////////////////////////////////////////////////////////////////
bool UniqueID::operator == (const UniqueID & x) const
{
	return (m_iLow == x.m_iLow) && (m_iHigh == x.m_iHigh);
}

//////////////////////////////////////////////////////////////////////////////
///
/// Inequality operator between two UniqueID objects
///
//////////////////////////////////////////////////////////////////////////////
bool UniqueID::operator != (const UniqueID & x) const
{
	return !(*this == x);
}

//////////////////////////////////////////////////////////////////////////////
///
/// Load/save object from/to a file
///
//////////////////////////////////////////////////////////////////////////////
void UniqueID::Serialize(FILE * hFile, bool bSave, void * pvArg)
{
	SerializeItem(hFile, bSave, &m_iLow);
	SerializeItem(hFile, bSave, &m_iHigh);
}

//////////////////////////////////////////////////////////////////////////////
///
/// Convert the UniqueID into a string suitable for storing in the registry
///
//////////////////////////////////////////////////////////////////////////////
wxString UniqueID::ToString() const
{
	return wxString::Format(wxT("%u,%u"), m_iLow, m_iHigh);
}

//////////////////////////////////////////////////////////////////////////////
///
/// Convert the output from ToString back into this UniqueID object
///
//////////////////////////////////////////////////////////////////////////////
void UniqueID::FromString(wxString strValue)
{
	unsigned long lTmp;

	strValue.BeforeFirst(wxT(',')).ToULong(&lTmp);
	m_iLow = lTmp;

	strValue.AfterFirst(wxT(',')).ToULong(&lTmp);
	m_iHigh = lTmp;
}
