/*
 *  LibRoadnav
 *  UniqueIDGenerator.cpp
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Contains the UniqueIDGenerator class - creates unique UniqueID objects
///
//////////////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef _MSC_VER
#pragma warning(disable: 4786)
#endif

#include <wx/wx.h>
#include <wx/filename.h>

#include "MapSupport.h"
#include "UniqueIDGenerator.h"


//////////////////////////////////////////////////////////////////////////////
///
/// \brief Constructor - load the last used ID from id.bin
///
//////////////////////////////////////////////////////////////////////////////
UniqueIDGenerator::UniqueIDGenerator()
{
	FILE * hFile;
	wxFileName fnID;

	fnID.AssignDir(GetDataPath());
	fnID.SetFullName(wxT("id.bin"));

	hFile = fopen(fnID.GetFullPath().mb_str(*wxConvCurrent), "rb");
	
	if (!hFile)
	{
		m_idCurrent.m_iLow = 0;
		m_idCurrent.m_iHigh = 0;
	}
	else
	{
		fread(&m_idCurrent, sizeof(m_idCurrent), 1, hFile);
		fclose(hFile);
	}
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Destructor - save the last used ID to id.bin
///
//////////////////////////////////////////////////////////////////////////////
UniqueIDGenerator::~UniqueIDGenerator()
{
	FILE * hFile;
	wxFileName fnID;

	fnID.AssignDir(GetDataPath());
	fnID.SetFullName(wxT("id.bin"));

	hFile = fopen(fnID.GetFullPath().mb_str(*wxConvCurrent), "wb");
	
	if (!hFile)
	{
		// uh oh
		return;
	}

	fwrite(&m_idCurrent, sizeof(m_idCurrent), 1, hFile);
	fclose(hFile);
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Generate a new UniqueID object by incrementing m_idCurrent and
/// returning it
///
//////////////////////////////////////////////////////////////////////////////
UniqueID UniqueIDGenerator::MakeID()
{
	m_idCurrent.m_iLow++;

	if (!m_idCurrent.m_iLow)
		m_idCurrent.m_iHigh++;

	return m_idCurrent;
}

