/*
 *  LibRoadnav
 *  UnitConversion.cpp
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Unit conversion functions - degrees to/from radians, feet to/from km,
/// etc.
///
//////////////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef HAVE_MATH_H
#include <math.h>
#endif

#include <wx/wx.h>

#include "UnitConversion.h"
#include "Constants.h"

//////////////////////////////////////////////////////////////////////////////
///
/// Converts from DMS to regular degrees
///
//////////////////////////////////////////////////////////////////////////////
double DMSToDecimal(short deg,unsigned short min,double sec)
{
   short sign = (deg < 0 ? -1 : 1);
   double angle = sign*(abs(deg) + double(min)/60.0 + sec/3600.);
   return angle;
}

//////////////////////////////////////////////////////////////////////////////
///
/// Converts from regular degrees to DMS
///
//////////////////////////////////////////////////////////////////////////////
void DecimalToDMS(double degrees, short & deg, unsigned short & min, double & sec)
{
   short sign = (degrees < 0 ? -1 : 1);
   degrees = fabs(degrees);

   deg = (short) floor(degrees);

   degrees -= deg;
   degrees *= 60;

   min = (unsigned short) floor(degrees);

   degrees -= min;
   sec = degrees*60;

   if ( sec == 60.0 )
   {
      sec = 0;
      min += 1;
   }

   if ( min == 60 )
   {
      min = 0;
      deg += 1;
   }

   deg *= sign;
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Convert from degrees to radians
///
//////////////////////////////////////////////////////////////////////////////
double DegreesToRadians(double x)
{
	return x / 180 * PI;
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Convert from radians to degrees
///
//////////////////////////////////////////////////////////////////////////////
double RadiansToDegrees(double x)
{
   return x * 180 / PI;
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Converts miles to kilometers
///
//////////////////////////////////////////////////////////////////////////////
double MilesToKilometers(double miles)
{
	return miles*1.609344;
}

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Converts feet to meters
///
//////////////////////////////////////////////////////////////////////////////
double FeetToMeters(double feet)
{
	return feet*0.3048;
}
