/*
 *  LibRoadnav
 *  Zip.cpp
 *
 *  Copyright (c) 2004 - 2008 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

//////////////////////////////////////////////////////////////////////////////
/// \file
///
/// Functions related to the handling of ZIP archives
///
//////////////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef _MSC_VER
#pragma warning(disable: 4786)
#endif

#include <wx/wx.h>
#include <wx/wfstream.h>
#include <wx/filesys.h>

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Unzip the specified zip file into the specified output directory
///
/// \param strZipFilename			Filename of zip file to unpack.
///
/// \param strOutputDirectory		Directory where zip file should be unpacked.
///
/// Currently zip files with directories in them are unsupported.
///
//////////////////////////////////////////////////////////////////////////////
void Unzip(wxString strZipFilename, wxString strOutputDirectory)
{
	wxFileSystem m_fsZip;
	wxString fnEntry;
	
	// find the first filename
	fnEntry = m_fsZip.FindFirst(strZipFilename + wxT("#zip:*"), 0);
	while(!fnEntry.IsEmpty())
	{
		wxFSFile * fsfEntry;
		wxInputStream * stmEntry;
		wxFileName fnOutput;
		wxString strOutput;
		
		// build up the output filename
		fnOutput.AssignDir(strOutputDirectory);
		if (!fnOutput.DirExists())
			fnOutput.Mkdir();
		fnOutput.SetFullName(fnEntry.Mid(strZipFilename.Length() + 5));

		// open the output file
		wxFileOutputStream stmOutput(fnOutput.GetFullPath());
		
		// open the input file
		fsfEntry = m_fsZip.OpenFile(fnEntry);
		
		if (fsfEntry)
		{
			stmEntry = fsfEntry->GetStream();
			
			if (stmEntry)
			{
				stmEntry->Read(stmOutput);
			}
			else
			{
				// error
			}

			delete fsfEntry;
		}
		else
		{
			// error
		}
		
		fnEntry = m_fsZip.FindNext();
	}
}
